import re
import requests
from mangapy import log
from mangapy.mangarepository import MangaRepository, Manga, Chapter, Page
from bs4 import BeautifulSoup


def unpack(p, a, c, k, e=None, d=None):
    def baseN(num, b, numerals="0123456789abcdefghijklmnopqrstuvwxyz"):
        return ((num == 0) and numerals[0]) or (baseN(num // b, b, numerals).lstrip(numerals[0]) + numerals[num % b])

    while (c):
        c -= 1
        if (k[c]):
            p = re.sub("\\b" + baseN(c, a) + "\\b",  k[c], p)
    return p


class FanFoxRepository(MangaRepository):
    name = "FanFox"
    base_url = "http://fanfox.net"
    proxies = None
    _session = None

    @property
    def session(self):
        if self._session is not None:
            return self._session

        self._session = requests.Session()
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=1.0,image/webp,image/apng,*/*;q=1.0',
            'Accept-Encoding': 'gzip, deflate',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/60.0.3112.101 Safari/537.36',
            'Accept-Language': 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3',
            'Referer': 'http://fanfox.net',
            'Connection': 'keep-alive'
        }

        self._session.cookies['isAdult'] = '1'
        self._session.headers = headers
        self._session.proxies = self.proxies
        return self._session

    def _get_chapters(self, list):
        manga_chapters = []
        if list is None:
            return manga_chapters            
        list_detail = list.find('ul', {'class': 'detail-main-list'})
        if list_detail is None:
            return manga_chapters
        chapters = list_detail.findAll('a', href=True)
        chapters_url = map(lambda c: c['href'], reversed(chapters))
         
        for url in chapters_url:
            number = url.split("/")[-2][1:]  # relative url, todo: regex
            absolute_url = "{0}{1}".format(self.base_url, url)
            number = float(number)
            chapter = FanFoxChapter(absolute_url, number, self.session)
            manga_chapters.append(chapter)

        return manga_chapters

    def search(self, manga_name) -> [Manga]:
        # support alphanumeric names with multiple words
        manga_name_adjusted = re.sub(r'[^A-Za-z0-9]+', '_', re.sub(r'^[^A-Za-z0-9]+|[^A-Za-z0-9]+$', '', manga_name)).lower()
        manga_url = "{0}/manga/{1}".format(self.base_url, manga_name_adjusted)
        response = self.session.get(url=manga_url)

        if response is None or response.status_code != 200:
            return None

        content = response.text
        soup = BeautifulSoup(content, features="html.parser")

        # list-2 contains all the chapters, list-1 only the last volume
        # in theory we should use list-1 and fallback to to list-2 if list-1 doesn't exist
        # but it seems that sometimes chapters are put in the wrong list
        # to solve this we merge both the lists
        list_two = soup.find('div', {'id': 'list-2'})
        list_one = soup.find('div', {'id': 'list-1'})

        if list_one is None and list_two is None:
            blocked = soup.find('p', {'class': 'detail-block-content'})
            if blocked:
                log.warning(blocked.getText())
            else:
                log.warning('No chapters found')
            return None

        list_one_chapters = self._get_chapters(list_one)
        list_two_chapters = self._get_chapters(list_two)
        chapters = list_one_chapters + list_two_chapters

        if not chapters:
            log.warning('No chapters list found')
            return None

        seen = set()
        unique_chapters = []
        for chapter in chapters:
            if chapter.number not in seen:
                unique_chapters.append(chapter)
                seen.add(chapter.number)
               
        sorted_chapters = sorted(unique_chapters, key=lambda chapter: chapter.number, reverse=False)

        manga = Manga(manga_name, sorted_chapters)
        return manga


class FanFoxChapter(Chapter):
    def __init__(self, first_page_url, number, session: requests.Session):
        self.first_page_url = first_page_url
        self.number = number
        self.session = session

    def _get_urls(self, content):
        js = re.search(r'eval\((function\b.+)\((\'[\w ].+)\)\)', content).group(0)
        encrypted = js.split('}(')[1][:-1]
        unpacked = eval('unpack(' + encrypted)
        return unpacked

    def _get_key(self, content):
        js = re.search(r'eval\((function\b.+)\((\'[\w ].+)\)\)', content).group(0)
        encrypted = js.split('}(')[1][:-1]
        unpacked = eval('unpack(' + encrypted)
        key_match = re.search(r'(?<=var guidkey=)(.*)(?=\';)', unpacked)
        key = key_match.group(1)
        key = key.replace('\'', '')
        key = key.replace('\\', '')
        key = key.replace('+', '')
        return key

    def _one_link_helper(self, content, page, base_url):
        cid = re.search(r'chapterid\s*=\s*(\d+)', content).group(1)
        key = self._get_key(content)
        final_url = '{}/chapterfun.ashx?cid={}&page={}&key={}'.format(base_url, cid, page, key)
        response = self.session.get(final_url)

        if response is None or response.status_code != 200:
            return None

        content = response.text
        return content

    def _parse_links(self, data):
        base_path = re.search(r'pix="(.+?)"', data).group(1)
        images = re.findall(r'"(/\w.+?)"', data)
        return [base_path + i for i in images]

    def pages(self) -> [Page]:
        base_url = self.first_page_url[:self.first_page_url.rfind('/')]
        response = self.session.get(self.first_page_url)

        if response is None or response.status_code != 200:
            return None

        content = response.text
        soup = BeautifulSoup(content, features="html.parser")
        page_numbers = soup.findAll("a", {"data-page": True})
        page_numbers = map(lambda x: int(x['data-page']), page_numbers)
        last_page_number = max(page_numbers)

        links = []
        for i in range(0, int(last_page_number / 2 + .5)):
            data = self._one_link_helper(soup.text, (i * 2) + 1, base_url)
            links += self._parse_links(self._get_urls(data))

        pages = []
        for i, link in enumerate(links):
            pages.append(Page(i, link))

        return pages


if __name__ == '__main__':
    repo = FanFoxRepository()
    manga = repo.search('naruto')
    assert manga is not None



import os
import re
import requests
import shutil
from functools import partial
from tqdm import tqdm
from PIL import Image
from urllib.parse import urlparse
from mangapy.mangarepository import Chapter, Page
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path


class ChapterArchiver(object):
    session = requests.Session()

    def __init__(self, path: str, max_workers=1):
        self.max_workers = max_workers
        self.path = Path(path).expanduser()

    def archive(self, chapter: Chapter, pdf: bool):
        if pdf:
            images_path = self.path.joinpath('.images')
        else:
            images_path = self.path.joinpath('images')

        isChapterNumberInt = isinstance(chapter.number, int) or chapter.number.is_integer()
        chapter_name = str(int(chapter.number)) if isChapterNumberInt else str(chapter.number)
        chapter_images_path = images_path.joinpath(str(chapter.number))
        chapter_images_path.mkdir(parents=True, exist_ok=True)
        pages = chapter.pages()
        description = ('Chapter {0}'.format(chapter_name))
        func = partial(self._save_image, chapter_images_path)  # currying

        if pdf:
            pdf_path = self.path.joinpath('pdf')
            chapter_pdf_file_path = pdf_path.joinpath(chapter_name + '.pdf')
            if os.path.isfile(chapter_pdf_file_path):
                print("⏺  {0} already downloaded and it will skipped.".format(chapter_name))
                return  # early exit if the file is already on disk

        with ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            list(tqdm(executor.map(func, pages), total=len(pages), desc=description, unit='pages', ncols=100))

        if pdf:
            pdf_path = self.path.joinpath('pdf')
            pdf_path.mkdir(parents=True, exist_ok=True)
            chapter_pdf_file_path = pdf_path.joinpath(chapter_name + '.pdf')
            self._create_chapter_pdf(chapter_images_path, chapter_pdf_file_path)
            shutil.rmtree(chapter_images_path)

    def _fetch_image(self, url: str):
        response = self.session.get(url, verify=False)
        if response.status_code != 200:
            return None
        return response.content

    def _save_image(self, image_path: Path, page: Page):
        file_name = str(page.number)
        image_url = page.url
        file_ext = urlparse(image_url).path.split('.')[-1]
        file_path = image_path.joinpath(file_name + '.' + file_ext)
        if os.path.isfile(file_path):
            return  # early exit if the file is already on disk

        if image_url.startswith('//'):
            image_url = 'http:' + image_url

        data = self._fetch_image(image_url)

        if data is None:
            return

        output = open(file_path, "wb")
        output.write(data)
        output.close()

    def _create_chapter_pdf(self, chapter_images_path: Path, pdf_path: Path):
        file_list = list(chapter_images_path.glob('**/*'))  # we don't care of their ext
        chapter_images_path = list(map(lambda path: str(path.absolute()), file_list))
        images_path = natural_sort(chapter_images_path)

        images = []
        for path in images_path:
            image = Image.open(path)
            image.load()
            if image.mode == 'RGBA':
                image = image.convert('RGB')
            images.append(image)

        images_count = len(images)
        if images_count <= 0:
            return
        elif images_count == 1:
            first_image = images.pop(0)
            first_image.save(pdf_path, "PDF", resolution=100.0, save_all=True)
        else:
            first_image = images.pop(0)
            first_image.save(pdf_path, "PDF", resolution=100.0, save_all=True, append_images=images)


def natural_sort(list):
    def convert(text):
        return int(text) if text.isdigit() else text.lower()

    def alphanum_key(key):
        return [convert(c) for c in re.split('([0-9]+)', key)]

    return sorted(list, key=alphanum_key)




import re
from abc import ABC, abstractmethod


class Manga(ABC):
    def __init__(self, title, chapters):
        self.title = title
        self.chapters = chapters

    @property
    def last_chapter(self):
        # latest chapter available
        return self.chapters[-1]

    @property
    def subdirectory(self):
        # subdirectory where chapters should be saved
        return re.sub(r'[^A-Za-z0-9]+', '_', re.sub(r'^[^A-Za-z0-9]+|[^A-Za-z0-9]+$', '', self.title)).lower()


class Page():
    def __init__(self, number: int, url: str):
        self.number = number
        self.url = url


class Chapter(ABC):
    def __init__(self, first_page_url: str, number: float):
        self.first_page_url = first_page_url
        self.number = number

    @abstractmethod
    def pages(self) -> [Page]:
        pass


class MangaRepository(ABC):
    base_url = None

    @abstractmethod
    def search(self, title) -> [Manga]:
        pass



import argparse
import yaml
import json
import logging
import os
import pkg_resources
import sys
from mangapy.mangapark import MangaParkRepository
from mangapy.fanfox import FanFoxRepository
from mangapy.chapter_archiver import ChapterArchiver
from mangapy import log
from pathlib import Path


version = pkg_resources.require("mangapy")[0].version
default_path_to_download_folder = str(os.path.join(Path.home(), "Downloads", "mangapy"))


def cmd_parse():
    """Returns parsed arguments from command line"""
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(help='Download modes.', dest='mode')

    yaml_parser = subparsers.add_parser("yaml")
    args_parser = subparsers.add_parser("title")

    yaml_parser.add_argument('yaml_file', type=str, help="Path to the .yaml file")

    args_parser.add_argument('manga_title', type=str, help="manga title to download")
    args_parser.add_argument('-s', '--source', type=str, help="manga source")
    args_parser.add_argument('-o', '--out', type=str, default=default_path_to_download_folder, help='download directory')
    args_parser.add_argument('-d', '--debug', action='store_true', help="set log to DEBUG level")
    args_parser.add_argument('--pdf', action='store_true', help="create a pdf for each chapter")

    args_parser.add_argument('-p', '--proxy', type=json.loads, help="use a proxy to download the chapters")
    group = args_parser.add_mutually_exclusive_group()
    group.add_argument('-a', '--all', action='store_true', help="download all chapters available")
    group.add_argument('-c', '--chapter', type=str, help="chapter(s) number to download")
    
    parser.add_argument('-v', '--version',
                        action='version',
                        version='{0} {1}'.format(parser.prog, version),
                        help="show program version and exit")

    args = parser.parse_args()
    return args


def main():
    args = cmd_parse()
    if args.mode == 'title':
        main_title(args)
    elif args.mode == 'yaml':
        main_yaml(args)


class MangaDownload:
    def __init__(self, **entries):
        self.__dict__.update(entries)

    def save_as_pdf(self) -> bool:
        if 'pdf' in self.__dict__.keys():
            return self.__dict__['pdf']
        return None

    def download_single(self) -> float:
        if 'download_single_chapter' in self.__dict__.keys():
            return float(self.__dict__['download_single_chapter'].strip())
        return None

    def download_last(self) -> bool:
        if 'download_last_chapter' in self.__dict__.keys():
            return self.__dict__['download_last_chapter']
        return False

    def download_all(self) -> bool:
        if 'download_all_chapters' in self.__dict__.keys():
            return self.__dict__['download_all_chapters']
        return False

    def download_range(self) -> (int, int):
        if 'download_chapters' in self.__dict__.keys():
            chapters = self.__dict__['download_chapters']
            chapters = chapters.split('-')
            begin = None
            end = None
            if len(chapters) == 2:
                begin = int(chapters[0])
                end = int(chapters[1]) if chapters[1] else None
            if begin and end and (int(begin) > int(end)):
                sys.exit('{0}: error: invalid chapter interval, the end should be bigger than start'.format(chapters))
            return begin, end
        return None


def main_yaml(args: argparse.Namespace):
    yaml_file = args.yaml_file.strip()

    try:
        with open(yaml_file, 'r') as f:
            dictionary = yaml.load(f, Loader=yaml.FullLoader)
            output = dictionary['output']

            proxy = None
            if 'proxy' in dictionary.keys() and dictionary['proxy']:
                proxy_info = dictionary['proxy']
                if 'http' and 'https' in proxy_info.keys():
                    print('Setting proxy')
                    proxy = dictionary['proxy']
                else:
                    print('The proxy is not in the right format and it will not be used.')

            debug_log = False
            if 'debug' in dictionary.keys() and dictionary['debug']:
                debug_log = True

            if 'fanfox' in dictionary.keys():
                for download in list(map(lambda manga: MangaDownload(**manga), dictionary['fanfox'])):
                    download.source = 'fanfox'
                    download.enable_debug_log = debug_log
                    download.output = output
                    download.proxy = proxy
                    start_download(download)
            
            if 'mangapark' in dictionary.keys():
                for download in list(map(lambda manga: MangaDownload(**manga), dictionary['mangapark'])):
                    download.source = 'mangapark'
                    download.enable_debug_log = debug_log
                    download.output = output
                    download.proxy = proxy
                    start_download(download)
    except Exception as error:   
        print(error)


def main_title(args: argparse.Namespace):
    download = MangaDownload()
    download.title = args.manga_title.strip()
    download.output = args.out.strip()
    download.pdf = args.pdf or False
    source = args.source

    if args.debug:
        download.enable_debug_log = True
    else:
        download.enable_debug_log = False

    if source is None:
        download.source = 'fanfox'
    else:
        download.source = source.strip().lower()

    download.proxy = None
    if args.proxy:
        if 'http' and 'https' in args.proxy.keys():
            print('Setting proxy')
            download.proxy = args.proxy
        else:
            print('The proxy is not in the right format and it will not be used.')
            
    if args.all:
        download.download_all_chapters = True

    elif args.chapter:
        chapters = args.chapter.split('-')
        if len(chapters) == 2:
            download.download_chapters = args.chapter
        else:
            download.download_single_chapter = args.chapter.strip()
    else:
        download.download_last_chapter = True

    start_download(download)


def start_download(download: MangaDownload):
    if download.enable_debug_log:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.ERROR)

    if download.source is None:
        repository = FanFoxRepository()
        repository_directory = 'fanfox'
        max_workers = 1  # to avoid bot detection
    else:
        source = download.source.strip().lower()
        if source == 'fanfox':
            repository = FanFoxRepository()
            repository_directory = source
            max_workers = 1  # to avoid bot detection
        elif source == 'mangapark':
            repository = MangaParkRepository()
            repository_directory = source
            max_workers = 5
        else:
            sys.exit('source is missing')

    if download.proxy:
        repository.proxies = download.proxy

    print('🔎  Searching for {0} in {1}...'.format(download.title, download.source))
    try:
        manga = repository.search(download.title)
    except Exception as e:
        logging.error(str(e))
        return

    if manga is None or len(manga.chapters) <= 0:
        print('❌  Manga {0} doesn\'t exist.'.format(download.title))
        return

    print('✅  {0} found'.format(manga.title))
    directory = os.path.join(download.output, repository_directory, manga.subdirectory)
    chapters = []

    if download.download_all():
        chapters = manga.chapters

    elif download.download_single() is not None:
        download_chapter = download.download_single()
        for chapter in manga.chapters:
            if chapter.number == download_chapter:
                chapters.append(chapter)
                break
        else:
            logging.error("❌  Chapter doesn't exist.")
            return

    elif download.download_range() is not None:
        range = download.download_range()
        range_begin = range[0]
        range_end = range[1]
        start = None
        stop = None
        for index, chapter in enumerate(manga.chapters):
            if chapter.number == range_begin:
                start = index
            if range_end and chapter.number == range_end:
                stop = index + 1
        for chapter in manga.chapters[start:stop]:
            chapters.append(chapter)
 
    else:  # manga._download_last()
        last_chapter = manga.last_chapter
        chapters.append(last_chapter)

    print('⬇️  Download started.')
    archiver = ChapterArchiver(directory, max_workers=max_workers)
    for chapter in chapters:
        try:
            archiver.archive(chapter, download.save_as_pdf())
        except Exception as e:
            logging.error(str(e))

    print('🎉  Download finished.')


if __name__ == '__main__':
    main_folder = os.getcwd()
    yaml_file = os.path.join(main_folder, 'sample3.yaml')
    sys.argv.insert(1, 'yaml')
    sys.argv.insert(2, yaml_file)

    # sys.argv.insert(1, 'title')
    # sys.argv.insert(2, 'jujutsu kaisen')
    # sys.argv.insert(3, '-o ~/Downloads/mangapy_test')
    # sys.argv.insert(4, '-c 1-100')
    # sys.argv.insert(5, '-s mangapark')
    # sys.argv.insert(6, '--pdf')
    # sys.argv.insert(7, '--debug')
    # sys.argv.insert(8, '-p {"http": "http://31.14.131.70:8080", "https": "http://31.14.131.70:8080"}')

    main()



"""Setup for the mangapy package."""

from setuptools import setup, find_packages

with open('README.md') as f:
    README = f.read()

setup(
    author="Alessandro Marzoli",
    author_email="me@alessandromarzoli.com",
    name='mangapy',
    license="MIT",
    description='Manga downloader',
    version='1.5.0',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/alemar11/mangapy',
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=['bs4', 'pillow', 'pyyaml', 'requests', 'tqdm'],
    classifiers=[
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Topic :: Games/Entertainment',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: End Users/Desktop',
    ],
    entry_points="""
    [console_scripts]
    mangapy = mangapy.cli:main
    """
)



import requests
import json
import re
import urllib3
from mangapy import log
from mangapy.mangarepository import MangaRepository, Manga, Chapter, Page
from bs4 import BeautifulSoup

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class MangaParkRepository(MangaRepository):
    name = "MangaPark"
    base_url = "https://mangapark.net"
    proxies = None
    _session = None

    @property
    def session(self):
        if self._session is not None:
            return self._session

        self._session = requests.Session()
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=1.0,image/webp,image/apng,*/*;q=1.0',
            'Accept-Encoding': 'gzip, deflate',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/60.0.3112.101 Safari/537.36',
            'Accept-Language': 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3',
            'Referer': 'https://mangapark.net/',
            'Connection': 'keep-alive'
        }
        self._session.cookies['set'] = 'theme=1&h=1&img_load=5&img_zoom=1&img_tool=1&twin_m=0' \
                                       '&twin_c=0&manga_a_warn=1&history=1&timezone=14'
        self._session.headers = headers
        self._session.proxies = self.proxies
        return self._session

    def search(self, title) -> [Manga]:
        manga_name_adjusted = re.sub(r'[^A-Za-z0-9]+', '-', re.sub(r'^[^A-Za-z0-9]+|[^A-Za-z0-9]+$', '', title)).lower()
        manga_url = "{0}/manga/{1}".format(self.base_url, manga_name_adjusted)
        response = self.session.get(url=manga_url, verify=False)
        if response is None or response.status_code != 200:
            return None

        text = response.text
        soup = BeautifulSoup(text, "html.parser")

        # this algorithm searches for the most updated stream and for the most content rich stream
        # if the 2 found streams are different, the two streams chapters will be merged
        # 1 fox
        # 3 panda
        # 6 rock
        # 4 duck
        # 101 mini
        streams = ['stream_1', 'stream_3', 'stream_6', 'stream_4', 'stream_101']
        contents = {}
        for stream in streams:
            content = soup.find('div', {'id': stream})
            if content is not None:
                list = self.parse_chapters(content)
                if list is not None:
                    log.info('content found for stream {0}'.format(stream))
                    contents[stream] = list
                else:
                    log.warning('content not found for stream {0}'.format(stream))

        if len(contents) == 0:
            log.warning('No chapters found')
            return None

        last_chapter_number = -1
        most_recenly_updated_stream = None
        number_of_chapters = -1
        most_rich_content_stream = None

        for stream, chapters in contents.items():
            max_chapter_number = max(chapter.number for chapter in chapters)
            if max_chapter_number is not None and max_chapter_number > last_chapter_number:
                last_chapter_number = max_chapter_number
                manga_chapters = chapters
                most_recenly_updated_stream = stream
            if most_rich_content_stream is not None:
                if len(chapters) > number_of_chapters:
                    number_of_chapters = len(chapters)
                    most_rich_content_stream = stream
            else:
                number_of_chapters = len(chapters)
                most_rich_content_stream = stream

        if most_recenly_updated_stream == most_rich_content_stream:
            log.info('using {0}'.format(most_recenly_updated_stream))
            manga_chapters = contents[most_recenly_updated_stream]
        else:
            # at this point we have 2 sources: one with the latest chapter and another one with probably the entire chapter list
            log.info('using {0} and {1}'.format(most_recenly_updated_stream, most_rich_content_stream))
            available_chapters = contents[most_recenly_updated_stream] + contents[most_rich_content_stream]
            seen = set()
            manga_chapters = []
            for chapter in available_chapters:
                if chapter.number not in seen:
                    manga_chapters.append(chapter)
                    seen.add(chapter.number)
               
            manga_chapters = sorted(manga_chapters, key=lambda chapter: chapter.number, reverse=False)

        manga = Manga(title, manga_chapters)
        return manga

    def parse_chapters(self, content):
        # parses all the chapter for a stream content
        # any minor version discovered (i.e. 11.4) will update the major version (i.e. 11)
        chapters_detail = content.select('a.ml-1')
        if chapters_detail is None:
            return None

        class Metadata:
            def __init__(self, url, title):
                self.url = url
                self.title = title

        manga_chapters = {}
        chapters_metadata = map(lambda c: Metadata(c['href'], c.string), reversed(chapters_detail))

        for metadata in chapters_metadata:
            # https://regex101.com/r/PFFb5l/10
            match = re.search(
                r'((?<=ch.)([0-9]*)|(?<=Chapter)\s*-?([0-9]*[.]?[0-9])|(?<=Page)\s*-?([0-9]*[.]?[0-9]))',
                metadata.title)
            if match is not None:
                try:
                    number = match.group(1) or 0
                    number = float(number)
                except ValueError:
                    number = 0
            else:
                number = 0

            if number == 0 and number in manga_chapters.keys():
                # some streams uses ch 0 in different volumes to identify side stories
                # i.e. Vol.23 Chapter 0: Side-A the sand
                # those chapter will be skipped
                log.warning('skipping chapter {0}'.format(metadata.title))
                continue

            url = metadata.url
            chapter_url = "{0}{1}".format(self.base_url, url)
            chapter = MangaParkChapter(chapter_url, abs(number))
            manga_chapters[number] = chapter

        sorted_chapters = sorted(manga_chapters.values(), key=lambda chapter: chapter.number, reverse=False)
        return sorted_chapters


class MangaParkChapter(Chapter):
    def pages(self) -> [Page]:
        response = requests.get(url=self.first_page_url, verify=False)
        pages = []

        if response is None or response.status_code != 200:
            return pages

        content = response.text
        soup = BeautifulSoup(content, "html.parser")
        scripts = soup.findAll('script')
        generator = (script for script in scripts if script.text.find('var _load_pages') > 0)
        for script in generator:
            match = re.search(r'(var _load_pages\s*=\s*)(.+)(?=;)', script.text)
            json_payload = match.group(2)
            json_pages = json.loads(json_payload)
            for page in json_pages:
                url = page['u']
                if url.startswith('//'):
                    url = 'https:' + page['u']
                pages.append(Page(page['n'], url))
        return pages


if __name__ == '__main__':
    repo = MangaParkRepository()
    manga = repo.search('naruto')
    assert manga is not None
